package de.elisabetheckstaedt.dipl.real;

import de.elisabetheckstaedt.dipl.common.Helper;

/**
 * Klasse reprsentiert Absorptionskltemaschinen (bisher nur einstufige LiBr)
 * @author Elisabeth Eckstdt
 */
public class AbKM extends Kaeltemaschine
{
	/**
	 * Q_0/Q_H [1] bei Nennlast
	 */
	private double zeta_K;
	
	private double[] Waermeverbrauch = new double[8760];

	/**
	 * reicht Parameter zum Vaterkonstruktor durch
	 * @param name
	 * @param rueckkuehler
	 * @param teillastregelung
	 */
	public AbKM(String name, Rueckkuehler rueckkuehler, String teillastregelung)
	{
		super(name, rueckkuehler, teillastregelung);
	}
	
	/**
	 * setzt Nennleistung, Faktoren nach VDI<p>
	 * ermittelt mit Vorlauftemeratur das zeta, ruft setZeta()<p>
	 * ermittelt die Abmessungen<p>
	 * ruft Investition()<p>
	 * @param nennleistung
	 * @param t_K_A
	 */
	public void init(double nennleistung, double t_K_A)
	{
		Nennleistung = nennleistung;
		Nutzungsdauer = 15; //a [VDI 2067:2007.03 Tab. A3 2.3.2.1.2]
		Instandsetzung = 1.5; //% der Investitionssumme pro a
		Wartung = 1; //% der Investitionssumme pro a
		Bedienung = 0; //h/a
		Hilfsantriebe = 0; //wre 0.07 laut [IUTA VI.18]
		Wasserfaktor = 0.004;
		Inspektion = 0;
		//zeta ermitteln nach DIN (Heiztemperaturniveau 130/110 angenommen)
		double zeta_6; 
		double zeta_14;
		if (Rueckkuehler instanceof Verdunstungsrueckkuehler)
		{
			zeta_6 = 0.71;
			zeta_14 = 0.73;
		}
		else
		{
			zeta_6 = 0.70;
			zeta_14 = 0.72;
		}
		setZeta_K(Helper.ggl_2pkt(t_K_A, 6, zeta_6, 14, zeta_14));
		laenge = 11/9100 * Nennleistung + 353/91; //VDI 3803 S.15
		hoehe = 4.39 * 0.0001 * Nennleistung + 178/91;
		breite = 3.29 * 0.0001 * Nennleistung + 713/455;
		Investition();
	}
	
	/**
	 * setzt Investition entsprechend Formel aus IUTA
	 */
	protected void Investition()
	{
		if (Nennleistung<50 || Nennleistung > 4750)
		{
			System.err.println("Fr Leistungen unter 50 und ber 4.750 kW ist keine Kostenfunktion hinterlegt." +
					" Sie wollten eine AbKM mit " + Nennleistung + " kW erzeugen");
		}
		Investition = (14740.2095 * Math.pow(Nennleistung,-0.6849) + 3.29) * Nennleistung; //IUTA S. VI.16
	}
	
	/**
	 * @param Kaelteleistung
	 * @param aussentemperatur
	 * setzt Kostenstelle.Stromverbrauch
	 * = aktuelle Klteleistung * Hilfsantriebsfaktor
	 */
	public void Stromverbrauch(double[] Kaelteleistung, double[] aussentemperatur)
	{
		for (int i=0; i<8760; i++)
		{
			Stromverbrauch[i] = Kaelteleistung[i] * Hilfsantriebe;
		}
	}
	
	/**
	 * ruft PLV()<p>
	 * setzt Stromverbraucher.Waermeverbrauch = Klteleistung * 1/(zeta + PLV)
	 * @param Kaelteleistung
	 * @param t_L
	 */
	public void Waermeverbrauch(double[] Kaelteleistung, double[] t_L)
	{
		double[] PLV;
		if (Kuehlmedium == "Wasser") PLV = PLV(Kaelteleistung);
		else PLV = PLV(t_L);
		for (int i=0; i<8760; i++)
		{
			Waermeverbrauch[i] = Kaelteleistung[i] * 1/(zeta_K * PLV[i]);
		}
	}
	
	/**
	 * @param Kaelteleistung
	 * @return Rckkhlleistung = Klteleistung * (1+1/zeta)
	 */
	public double[] Rueckkuehlleistung (double[] Kaelteleistung)
	{
		double[] Rueckkuehlleistung = new double[8760];
		for (int i=0; i<8760; i++)
		{
			Rueckkuehlleistung[i] = Kaelteleistung[i] * (1 + 1 / (zeta_K));// * f_1i(Kaelteleistung[i])));
		}
		return Rueckkuehlleistung;
	}
	
	/**
	 * @param t_w_a Khlwasseraustrittstemperatur [C]
	 * @return Teillastfaktor der Rckkhlung in dem Zeitschritt mit dieser Khlwasseraustrittstemperatur
	 */
	public double f_2i(double t_w_a) {
		double untergrenze;
		double obergrenze;
		double m;
		double n;
		if (Rueckkuehler instanceof Trockenrueckkuehler)
		{
			untergrenze = 20;
			obergrenze = 50;
			m = -0.0294;
			n = 2.3235;
		}
		else //Verdunstungsrckhler
		{
			untergrenze = 20;
			obergrenze = 40;
			m = -0.0217;
			n = 1.7174;
		}
		if (t_w_a<untergrenze || t_w_a >obergrenze) System.err.println("Fr diese Khlwasseraustrittstemperatur ist kein " +
				"Teillastfaktor hinterlegt (AbKM.f_2i(t_W_A)) t_W_A=" + t_w_a);
		double f_2i = m * t_w_a + n;
		return f_2i;
	}

	/**
	 * ruft RK.setNennleistung(), Investition()
	 * @param nennleistung
	 */
	public void setNennleistung(double nennleistung)
	{
		Nennleistung = nennleistung;
		Rueckkuehler.setNennleistung(Nennleistung * (1+1/zeta_K));
		Investition();
	}

	/**
	 * @param zeta_k
	 * ruft RK.setNennleistung()
	 */
	public void setZeta_K(double zeta_k) {
		
		this.zeta_K = zeta_k;
		Rueckkuehler.setNennleistung(Nennleistung * (1+1/zeta_K));
	}

	public String Parametertext()
	{
		return "zeta_K: " + zeta_K + " [1] Wasserfaktor: " + Wasserfaktor + " [m/kWh_Klte] Hilfsantriebe: " 
		+ Hilfsantriebe + " [kWh_Strom/kWh_Klte]";
	}
	
	public String getBauteilart()
	{
		return "AbKM";
	}

	public double[] getWaermeverbrauch() {
		return Waermeverbrauch;
	}

	public double getZeta_K()
	{
		return zeta_K;
	}

}
