package de.elisabetheckstaedt.dipl.real;
//
import net.tinyos.matlab.MatClient;
import de.elisabetheckstaedt.dipl.common.Einzelobjekt;
import de.elisabetheckstaedt.dipl.common.Helper;
import de.elisabetheckstaedt.dipl.common.Versorgungsaufgabe;


/**
 * reprsentiert alle Rohre mit Formstcken und Einbauten
 * @author Elisabeth Eckstdt
 */
public class Rohrnetz extends Einzelobjekt{
	
	/**
	 * [m]
	 */
	private double rohrlaenge;
	/**
	 * [m]
	 */
	private double trassenlaenge;
	/**
	 * [mm]
	 */
	private double durchmesser;
	/**
	 * Rohrreibungszahl [1]
	 */
	private double lambda;
	/**
	 * [1]
	 */
	private double einbautenfaktor;
	/**
	 * bisher KMR (und Stahl) implementiert
	 */
	private final String material;
	
	/**
	 * [/m] fr Rohr 
	 */
	private double rohrkosten;
	/**
	 * [/m] fr die Trasse
	 */
	private double trassenkosten;
	
	/**
	 * @param Name
	 * @param material (KMR, Stahl)
	 */
	public Rohrnetz(String Name, String material) {
		super(Name);
		this.material = material;
	}
	
	/**
	 * Minimale Rohrlnge zur Verbindung aller Verbraucher und der Zentrale (TSP, Stern) in m<p>
	 * setzt DUrchmesser = 500, lambda = 0.25, Einbautenfaktor = 1.5, Verlegelnge = halbe Rohrlnge<p>
	 * ruft init(mit allen Parametern)
	 * @param ort_kz Ort der Kltezentrale
	 * @param verbraucher
	 */
	public void init(double[] ort_kz, Verbraucher[] verbraucher)
	{
		if (verbraucher.length == 1)
		{
			if (verbraucher[0].getOrt()[0] == ort_kz[0] && verbraucher[0].getOrt()[1] == ort_kz[1])
			{
				rohrlaenge = 0;
			}
			else
			{
				rohrlaenge = 2 * Math.sqrt(
						Math.pow((verbraucher[0].getOrt(0)-ort_kz[0]),2)
						+Math.pow((verbraucher[0].getOrt(1)-ort_kz[1]),2));
			}
		}
		else
		{
			Versorgungsaufgabe VA = new Versorgungsaufgabe(verbraucher);
			double[][] Koordinaten = VA.Koordinaten();
			double stern = 0;
			double[][] KoordinatenmZ = new double[2][verbraucher.length+1];
			for (int i=0; i<verbraucher.length; i++)
			{
				stern = stern + 2 * Math.sqrt(Math.pow((Koordinaten[0][i]-ort_kz[0]),2)+Math.pow((Koordinaten[1][i]-ort_kz[1]),2));
				KoordinatenmZ[0][i] = Koordinaten[0][i];
				KoordinatenmZ[1][i] = Koordinaten[1][i];
			}
			KoordinatenmZ[0][verbraucher.length] = ort_kz[0];
			KoordinatenmZ[1][verbraucher.length] = ort_kz[1];
			Object[] arg = {KoordinatenmZ,new Double(2000),new Double(0.97),new Double(5000),new Double(2)};
			try
			{
				MatClient mc = new MatClient();
				Object tspo = mc.sendCommand("TSP", arg);
				double tsp = ((double[]) tspo)[0];
				rohrlaenge = Math.min(2 * tsp,stern);
			}
			catch (Exception e)
			{
				System.err.println("Konnte keine Verbindung zu Matlab aufbauen. Rohrverlegung wird als Stern angenommen.");
				rohrlaenge = stern;
			}
		}
		init(rohrlaenge, rohrlaenge/2, 500, 0.25, 1.5);
	}
	
	/**
	 * ruft setDurchmesser()
	 * @param rohrlaenge
	 * @param verlegelaenge
	 * @param durchmesser
	 * @param lambda
	 * @param einbautenfaktor
	 */
	public void init(double rohrlaenge, double verlegelaenge, double durchmesser, 
			double lambda, double einbautenfaktor)
	{
		Nutzungsdauer = 40; //a [VDI 2067:2007.03 Tab. A3 2.2.2]
		Instandsetzung = 1; //% der Investitionssumme pro a
		Wartung = 0; //% der Investitionssumme pro a
		Bedienung = 0; //h/a
		Inspektion = 0;
		this.rohrlaenge = rohrlaenge;
		this.trassenlaenge = verlegelaenge;
		setDurchmesser(durchmesser);
		this.lambda = lambda;
		this.einbautenfaktor = einbautenfaktor;
	}


	/**
	 * setzt Kosten = rohrkosten * rohrlaenge + trassenkosten * trassenlaenge)
	 */
	private void Investition()
	{
		Investition = rohrkosten * rohrlaenge + trassenkosten * trassenlaenge;
	}

	/**
	 * liest lambda, rohrlnge, drchmesser, einbautenfaktor
	 * @param Vp Volumenstromvektor [m/s]
	 * @param rho [g/cm]
	 * @return Druckverlust des Rohrnetzes [Pa]
	 */
	public double[] Druckverlust(double[] Vp, double rho)
	{
		double zeta = lambda * rohrlaenge * 1000 / durchmesser; //[1]
		double[] delta_p = new double[8760];
		double[] u = new double[8760];
		for (int i=0; i<8760; i++)
		{
			u[i] = Vp[i] / (Math.PI / 4 * Math.pow(durchmesser, 2)) * Math.pow(10, 6); //in m/s
			delta_p[i] = zeta * rho / 2 * Math.pow(u[i], 2) * 1000 * einbautenfaktor; //in Pa
		}
		return delta_p;
	}

	/**
	 * setzt rohrkosten und trassenkosten je nach Material; ruft Investition()
	 * @param durchmesser
	 * 
	 */
	public void setDurchmesser(double durchmesser) {
		this.durchmesser = durchmesser;
		double Schachtbreite = durchmesser / 1000 + 0.9; //[m]
		double Schachttiefe = 1.2 + durchmesser / 1000 + 0.3; //frostfreie Verlegung + Rohr + Bettung [m]
		double Erdaushub = Schachtbreite * Schachttiefe; //m/m
		double Erdbewegungskosten = Erdaushub * (12 + 0.2 * 7 + 0.8 * 10); //[] Aushub + Wiedereinbringen * Neueinbringen
		double Oberflaechenkosten = Schachtbreite * 150; //[] Bitumenstrae aufbringen
		trassenkosten = Erdbewegungskosten + Oberflaechenkosten;
		if (material == "KMR")
		{
			rohrkosten = (2.7751 * durchmesser + 207.9);
		}
		else if (material == "Stahl")
		{
			double rohre = (102 + 67); //Rohre + Formteile
			double unterstuetzngskonstruktion = 49; //TODO gehrt eigentlich anderswo hin
			rohrkosten = rohre + unterstuetzngskonstruktion;
		}
		else if (material == "PE")
		{
			rohrkosten = 70 + 246; //Rohr + Einbauteile
		}
		Investition();
	}

	public void setEinbautenfaktor(double einbautenfaktor) {
		this.einbautenfaktor = einbautenfaktor;
	}

	public void setLambda(double lambda) {
		this.lambda = lambda;
	}

	/**
	 *  ruft Investition()
	 * @param verlegelaenge
	 */
	public void setTrassenlaenge(double verlegelaenge) {
		this.trassenlaenge = verlegelaenge;
		Investition();
	}

	/**
	 * ruft Investition()
	 * @param rohrlaenge
	 */
	public void setRohrlaenge(double rohrlaenge) {
		this.rohrlaenge = rohrlaenge;
		Investition();
	}

	public String Investitionstext()
	{
		return super.Investitionstext() + " Rohrlnge: " + Helper.Zahlformat(rohrlaenge,0) + " m" + 
		" Verlegelnge: " + Helper.Zahlformat(trassenlaenge,0) + " m" + 
		" Durchmesser: " + Helper.Zahlformat(durchmesser,0) + " mm" + 
		" Material: " + material;
	}

	public String Parametertext()
	{
		return "Einbautenfaktor: " + einbautenfaktor + " [1] lambda: " + lambda + " [1]";
	}

	public String getBauteilart()
	{
		return "Rohrnetz";
	}

	public double getDurchmesser() {
		return durchmesser;
	}

	public double getRohrlaenge() {
		return rohrlaenge;
	}

	public double getEinbautenfaktor() {
		return einbautenfaktor;
	}

	public double getLambda() {
		return lambda;
	}

	public double getTrassenlaenge() {
		return trassenlaenge;
	}

	public double getRohrkosten() {
		return rohrkosten;
	}

	/**
	 * ruft Investition()
	 * @param rohrkosten
	 */
	public void setRohrkosten(double rohrkosten) {
		this.rohrkosten = rohrkosten;
		Investition();
	}

	public double getTrassenkosten() {
		return trassenkosten;
	}

	/**
	 * ruft Investition()
	 * @param trassenkosten
	 */
	public void setTrassenkosten(double trassenkosten) {
		this.trassenkosten = trassenkosten;
		Investition();
	}
}