package de.elisabetheckstaedt.dipl.real;

import java.util.Arrays;

import de.elisabetheckstaedt.dipl.common.Helper;
import de.elisabetheckstaedt.dipl.common.Klimadatensatz;
import de.elisabetheckstaedt.dipl.common.Sammelobjekt;
import de.elisabetheckstaedt.dipl.common.Versorgungsaufgabe;

/**
 * Untergruppierung des Gesamtsystems, charakterisiert durch eine gemeinsame Kltezentrale
 * @author Elisabeth Eckstdt
 */
public class Teilsystem extends Sammelobjekt {

	/**
	 * Vektor mit 2 Elementen: X & Y-Koordinate [m]
	 */
	private double[] Ort = new double[2];
	
	/**
	 * Vektor, der alle in der Kltezentrale enthaltenen Kltemaschinen enthlt
	 */
	private final Kaeltemaschine[] Kaeltemaschinen;
	
	/**
	 * Vektor, der alle der Kltezentrale zugeordneten Rueckkuehler enthlt
	 */
	final private Rueckkuehler[] Rueckkuehler;
	
	/**
	 * Matrix Kltemaschinenanzahlx8760 gibt zu jeder Stunde den Anteil jeder Kltemaschine an der Klterzeugung an [1]
	 */
	private final double[][] Leistungsaufteilung_KM;
	
	/**
	 * Vektor enthlt alle dem Teilsystem zugeordneten Subnetze
	 */
	final private Subnetz[] subnetze;

	/**
	 * Zu liefernde Vorlauftemperatur, wird fr jedes Teilsystem einmal festgelegt und ist dann konstant
	 * Kaltwasservorlauf [C]
	 */
	private final double t_K_A;
	
	private final Verbraucher[] verbraucher;
	
	private final Gebaeude gebaeude;
	
	private final MSR msr;
	
	private final Armatur armaturen;
	
	private final Pumpe[] pumpen_rk;
	
	private final Pumpe[] pumpen_km;
	
	private final Rohrnetz verrohrung;
	
	/**
	 * Vorlauftemperatur aufs Minimum aller Verbraucher gesetzt<p>
	 * restliche Bestandteile werden erzeugt
	 * @param Name
	 * @param kaeltemaschinen
	 * @param leistungsaufteilung_KM
	 * @param subnetze
	 */
	public Teilsystem(String Name, final Kaeltemaschine[] kaeltemaschinen, 
			final double[][] leistungsaufteilung_KM, final Subnetz[] subnetze) {
		super(Name);
		Kaeltemaschinen = kaeltemaschinen;
		Leistungsaufteilung_KM = leistungsaufteilung_KM;
		
		//Rckkhlerliste erstellen
		Rueckkuehler = new Rueckkuehler[Kaeltemaschinen.length]; 
		for (int i=0; i<Kaeltemaschinen.length ; i++)
		{
			Rueckkuehler[i] = Kaeltemaschinen[i].Rueckkuehler;
		}
		
		this.subnetze = subnetze;
		
		//Verbraucherliste erstellen
		int verbraucherzahl = 0;
		for (int i=0; i<subnetze.length; i++)
		{
			for (int j=0; j<subnetze[i].getVerbraucher().length; j++)
			{
				verbraucherzahl++;
			}
		}
		verbraucher = new Verbraucher[verbraucherzahl];
		int k=0;
		for (int i=0; i<subnetze.length; i++)
		{
			for (int j=0; j<subnetze[i].getVerbraucher().length; j++)
			{
				verbraucher[k] = subnetze[i].getVerbraucher()[j];
				k++;
			}
		}
		
		//Vorlauftemperatur setzen
		double minimaltemp = 100;
		for (int i=0; i<verbraucher.length; i++)
		{
			if (verbraucher[i].getVorlauftemperatur() < minimaltemp)
			{
				minimaltemp = verbraucher[i].getVorlauftemperatur();
			}
		}
		t_K_A = minimaltemp;
		gebaeude = new Gebaeude("Gebude " + Name);
		msr = new MSR("MSR " + Name);
		armaturen = new Armatur("Armaturen " + Name);
		verrohrung = new Rohrnetz("Verrohrung " + Name, "Stahl");
		pumpen_rk = new Pumpe[1];
		pumpen_rk[0] = new Pumpe("Rckkhlerpumpen " + Name);
		pumpen_km = new Pumpe[1];
		pumpen_km[0] = new Pumpe("Kltemaschinenpumpen " + Name);
	}

	/**
	 * Ort auf Mitte aller Verbraucher setzen<p>
	 * initialisiert Pumpen in Zentrale, Verrohrung, Gebude, MSR, Armaturen<p>
	 * setzt deren aller Investition auf null, wenn es keine Zentrale im eigentlichen Sinne 
	 * ist (nur ein Verbraucher angeschlossen)<p>
	 * @param Aussentemperatur
	 */
	public void init(double[] Aussentemperatur)
	{
		Versorgungsaufgabe VA = new Versorgungsaufgabe(verbraucher, new Klimadatensatz(Aussentemperatur));
		double XMitte = Helper.sum(VA.Koordinaten()[0])/VA.Koordinaten()[0].length;
		double YMitte = Helper.sum(VA.Koordinaten()[1])/VA.Koordinaten()[1].length;
		Ort[0] = XMitte;
		Ort[1] = YMitte;
		verrohrung.init(500, 0, 80, 0.25, 2.5); //im Beispiel waren es 571m
		pumpen_km[0].init(Helper.max(VA.Volumenstrom(new Wasser(), t_K_A)), 2);
		pumpen_rk[0].init(2 * Helper.max(VA.Volumenstrom(new Wasser(), t_K_A)), 2); //nherungsweise doppelte Frdermenge wie KM Kreislauf
		gebaeude.init(Kaeltemaschinen);
		double kostenkmrk = 0;
		boolean nurkompakt = true;
		for (int i = 0; i<Kaeltemaschinen.length; i++)
		{
			kostenkmrk = kostenkmrk + Kaeltemaschinen[i].getInvestition();
			nurkompakt = nurkompakt && (Kaeltemaschinen[i] instanceof Kompaktanlage);
		}
		for (int i = 0; i<Rueckkuehler.length; i++)
		{
			kostenkmrk = kostenkmrk + Rueckkuehler[i].getInvestition();
		}
		msr.init(kostenkmrk * 0.25);
		armaturen.init(4800 + kostenkmrk * 0.095);
		if (nurkompakt) //Nur Luftrckkhler
		{
			pumpen_rk[0].setInvestition(0);
			if (Kaeltemaschinen.length == 1) //keine KZ im eigentlichen Sinne
			{
				msr.setInvestition(0);
				armaturen.setInvestition(0);
				gebaeude.setInvestition(0);
				verrohrung.setRohrlaenge(0);
				pumpen_km[0].setInvestition(0);
			}
		}
	}

	/**
	 * ruft von allen Kindobjekten  die Verbrauchsberechnungsmethoden<p>
	 * setzt den Stromverbrauch der KZ-internen Pumpen erstmal auf null
	 * @param versorgungsaufgabe
	 */
	public void verbraeucheBerechnen(Versorgungsaufgabe versorgungsaufgabe) //soll auch mal mit anderere als der Auslegungsaufgabe mglich sein
	{
		double[] t_L = versorgungsaufgabe.getKlimadatensatz().getAussentemperatur();
		gebaeude.Stromverbrauch();
		msr.Stromverbrauch();
		double[] nullarray = new double[8760];
		Arrays.fill(nullarray, 0);
		pumpen_km[0].Stromverbrauch(nullarray,nullarray);
		pumpen_rk[0].Stromverbrauch(nullarray,nullarray);
		for (int i=0; i<Kaeltemaschinen.length; i++)
		{
			Kaeltemaschinen[i].Wasserverbrauch(Kaelteleistungen(i, t_L));
			Kaeltemaschinen[i].stromGes(Kaelteleistungen(i, t_L), versorgungsaufgabe.getKlimadatensatz()); //setzt KM und RK Strom
			if (Kaeltemaschinen[i] instanceof AbKM)
			{
				((AbKM) Kaeltemaschinen[i]).Waermeverbrauch(Kaelteleistungen(i, t_L), t_L);
			}	
		}
		for (int i=0; i<subnetze.length; i++) //Pumpenstrom
		{
			subnetze[i].getPumpen().Stromverbrauch(
					subnetze[i].volumenstrom(versorgungsaufgabe.getKlimadatensatz().getAussentemperatur(),t_K_A),
					subnetze[i].Druckverlust(
							subnetze[i].volumenstrom(versorgungsaufgabe.getKlimadatensatz().getAussentemperatur(), t_K_A)));
		}
		
	}

	/**
	 * Summe ber alle subnetze<p>
	 * ruft Versorgungsaufgabe.Lastgang()<p>
	 * liest subnetze<p>
	 * @see Versorgungsaufgabe#Lastgang()
	 * @param T_A
	 * @return [kW]
	 */
	public double[] Kaelteleistung(double[] T_A)
	{
		double[] KL = new double[8760];
		Arrays.fill(KL, 0);
		Versorgungsaufgabe dummy = new Versorgungsaufgabe(new Klimadatensatz(T_A));
		
		for (int i=0; i<subnetze.length; i++)
		{
			dummy.setVerbraucher(subnetze[i].getVerbraucher());
			double[] TNL = dummy.Lastgang();
			for (int j=0; j<8760; j++)
			{
				KL[j] = KL[j] + TNL[j];
			}
			//TODO + Speicherbeladung
		}
		return KL;
	}

	/**
	 * liest {@link Teilsystem#Leistungsaufteilung_KM}<p>
	 * ruft {@link Teilsystem#Kaelteleistung(T_A)}
	 * @param i Nummer der Kltemaschine
	 * @return Vektor 8760, Kaelteleistung der Kaeltemaschine mit Index i zu jeder Jahresstunde
	 */
	public double[] Kaelteleistungen(int i, double[] T_A)
	{
		double[] KaelteleistungKZ = Kaelteleistung(T_A);
		double[] LA = Leistungsaufteilung_KM[i];
		double[] KaelteleistungKM = new double[8760];
		for (int j=0; j<8760; j++)
		{
			KaelteleistungKM[j] = KaelteleistungKZ[j] * LA[j];
		}
		return KaelteleistungKM;
	}

	/**
	 * @param ort
	 * @param T_A Aussentemperatur fr Pumpe
	 */
	public void setOrt(double[] ort, double[] T_A) {
		Ort = ort;
		for (int i=0; i<subnetze.length; i++)
		{
			subnetze[i].getRohrnetz().init(ort, subnetze[i].getVerbraucher());
			subnetze[i].getPumpen().setFoerderhoehe(subnetze[i]);
		}
	}

	public String Parametertext()
	{
		return "Ort: " + Helper.Zahlformat(Ort[0],0) + "/" + Helper.Zahlformat(Ort[1],0) + " [m]";
	}

	public Object[] getStruktur()
	{
		Object[] versammelt = new Object[9];
		versammelt[0] = Kaeltemaschinen;
		versammelt[1] = Rueckkuehler;
		versammelt[2] = subnetze;
		Gebaeude[] g = {gebaeude};
		versammelt[3] = g;
		MSR[] m = {msr};
		versammelt[4] = m;
		Armatur[] a = {armaturen};
		versammelt[5] = a;
		versammelt[6] = pumpen_rk;
		versammelt[7] = pumpen_km;
		Rohrnetz[] r = {verrohrung};
		versammelt[8] = r;
		return versammelt;
	}
	public String getBauteilart()
	{
		return "Teilsystem";
	}

	public Kaeltemaschine[] getKaeltemaschinen() {
		return Kaeltemaschinen;
	}
	
	public Rueckkuehler[] getRueckkuehler() {
		return Rueckkuehler;
	}

	public double[] getOrt() {
		return Ort;
	}

	public double[][] getLeistungsaufteilung_KM() {
		return Leistungsaufteilung_KM;
	}
	
	public Subnetz[] getSubnetze() {
		return subnetze;
	}

	public double getT_K_A() {
		return t_K_A;
	}
	public Armatur getArmaturen() {
		return armaturen;
	}

	public Gebaeude getGebaeude() {
		return gebaeude;
	}

	public MSR getMsr() {
		return msr;
	}

	public Pumpe[] getPumpen_km() {
		return pumpen_km;
	}

	public Pumpe[] getPumpen_rk() {
		return pumpen_rk;
	}

	public Verbraucher[] getVerbraucher() {
		return verbraucher;
	}

	public Rohrnetz getVerrohrung() {
		return verrohrung;
	}

}
